@@ -1,28 +1,27 @@
+#!/usr/bin/env python
+''''A useful collection of helper functions for SWS'''
 import os
-import glob
 import imp
-from copy import copy,deepcopy
 
 import numpy as np
 from scipy import ndimage
-import wx
 import Image
 import matplotlib.pyplot as plt
 
-from cmpGen import cmpGen
 import FilenameSort as FS
-import ExcelHelper
 import ImageContour
-from ValueReceived import imshow_vr
 import GifTiffLoader as GTL
-import SeedWaterSegmenter as SWS
+#import SeedWaterSegmenter as SWS # No longer needed; plus SWS imports this (and circluar references are bad)
+import mahotas
 
-from np_utils import flatten,limitInteriorPoints,totuple,removeDuplicates,deletecases,floatIntStringOrNone
+#from list_utils:
+from np_utils import deletecases,floatIntStringOrNone
 
 #from MyPath import cprint
 #from pylab import *
 
 def norm(a,framesExcludeAfter=None):
+    '''Standard norm (divide by the mean). Can ignore frames at the end when computing the mean'''
     if framesExcludeAfter==None:
         return a/np.mean(a)
     else:
@@ -34,7 +33,9 @@ def openCSV(filename,colDelim=',',rowDelim='\n',allowNone=True):
     r=fid.read()
     fid.close()
     
-    return [[floatIntStringOrNone(i) for i in j.split(colDelim)] for j in r.replace('\r','').replace(' ','').split(rowDelim)]
+    return [[floatIntStringOrNone(i)
+              for i in j.split(colDelim)]
+             for j in r.replace('\r','').replace(' ','').split(rowDelim)]
 
 def GetWoutline(watershed,dilate):
     woutline = np.zeros(watershed.shape,np.int)
@@ -45,7 +46,7 @@ def GetWoutline(watershed,dilate):
     
     return woutline
 
-def GetSeedArray(watershed,SeedsPyFile,frameNumber):
+def GetSeedArray(watershed,SeedsPyFile,frameNumber):  # THIS FUNCTION IS CURRENTLY BROKEN
     # Load the SeedsPyFile:
     fid = open(SeedsPyFile)
     try:
@@ -58,7 +59,9 @@ def GetSeedArray(watershed,SeedsPyFile,frameNumber):
     seedArray = np.zeros(watershed.shape,np.int)
     
     if seedList[frameNumber]!=None:
-        SWS.PointsToArray(seedList[frameNumber], seedVals[frameNumber],seedArray)
+        pass
+        ################## This function no longer exists:
+        ##################SWS.PointsToArray(seedList[frameNumber], seedVals[frameNumber],seedArray)
     val = 2 #(2 if self.walgorithm=='cv' else 1) # I could do this, but what's the need?
     seedArray[:val,:]  = 1
     seedArray[-val:,:] = 1
@@ -125,11 +128,11 @@ def CreateMapPlots(watershed):
     return rgbM
 def CreateDiffImage(arr,targetArr):
     return ((arr - targetArr)!=0).astype(np.uint8)
-def GetTimeAxisAdj():
+def GetTimeAxisAdj(SegmentsDirectory):
     sList = []
     timeAdj = 0
     count=0
-    for i in FS.getSortedListOfFiles(di,'*.tif'):
+    for i in FS.getSortedListOfFiles(SegmentsDirectory,'*.tif'):
         count+=1
         d,h,m,s=map(int,os.path.split(i)[1].split('_')[2:6])
         h += d*24
@@ -140,13 +143,12 @@ def GetTimeAxisAdj():
                 print count
                 timeAdj += sList[-1] - s + 80
                 s = sList[-1] + 80
-            pass
         sList.append(s)
     return sList
 
 def SaveOverlayImages(tifStackData,SeedsPyFile,SegmentsDirectory,red=True):
     # This function is more-or-less just a template right now, has MAJOR issues...
-    r = GTL.LoadFileSequence(di, wildcard='*.tif')
+    r = GTL.LoadFileSequence(SegmentsDirectory, wildcard='*.tif')
     rgbM = CreateMapPlots(r)
     
     for i in range(len(Files)):
@@ -221,6 +223,23 @@ def polyArea(vertexList):
     a=np.array(vertexList,dtype=np.float)
     return abs(np.sum(np.cross(a,np.roll(a,-1,axis=0)))/2)
 
+def ConvertOutlinesToWatershed(origArr,outlines,useDilate=False,structure=[[0,1,0],[1,1,1],[0,1,0]]):
+    if useDilate:
+        outlines=ndimage.morphology.binary_dilation(origArr,structure=structure)
+    labels = ndimage.label(1-outlines)[0]
+    wh=np.where(labels==0)
+    labels[wh]=-1
+    labels+=1
+    labels = labels.astype(np.uint16)
+    labels[0]=1
+    labels[-1]=1
+    labels[:,0]=1
+    labels[:,-1]=1
+    
+    water = mahotas.cwatershed(origArr,labels)
+    return water
+
+
 def CheckForMalformedRegions(watershed,usePrint=True):
     '''This goes through all frames and values and uses shapely to test if regions are disjoint in any way'''
     
@@ -232,8 +251,11 @@ def CheckForMalformedRegions(watershed,usePrint=True):
     outputString = ''
     
     for frame in range(len(watershed)):
+        print 'Checking Frame',frame
         if watershed[frame]!=None:
             allVals = np.unique(watershed[frame])
+            if len(allVals)<2:
+                continue # no point in checking if it's empty or just a big solid rectangle...
             for v in allVals:
                 if v==1:
                     # For the background, take the inverse instead
@@ -288,7 +310,7 @@ def GetWoundNeighborSetsByFrame(neighborPairsByFrame,wv):
             if (n[0] in woundNeighbors) and (n[1] in woundNeighbors):
                 woundNeighbors2.append(tuple(sorted(n)))
         secondNeighbors = [] # Anything touching a wound neighbor
-        woundNeighborOuterPairs = [] # This essentially constitutes everything that would make up the perimeter around the 1st ring of wound neighbors
+        woundNeighborOuterPairs = [] # Everything making up the perimeter around 1st ring of wound neighbors
         for n in nPairs:
             if (n[0] in woundNeighbors) and not (n[1] in (woundNeighbors+[wv])):
                 secondNeighbors.append(n[1])
@@ -304,189 +326,6 @@ def GetWoundNeighborSetsByFrame(neighborPairsByFrame,wv):
         woundNeighborOuterPairsByFrame.append(woundNeighborOuterPairs)
     return woundNeighborsByFrame,woundNeighbors2ByFrame,secondNeighborsByFrame,woundNeighborOuterPairsByFrame
 
-def GetContourValuesLengthsAndSubContoursByFrame(watershed,allValsByFrame):
-    cVLSByFrame = []
-    for frame in range(len(watershed)):
-        cVLS = [] # for each subcountour: [value, length, subcontour points]
-        for v in allValsByFrame[frame]:
-            boundingRect=ImageContour.GetBoundingRect(watershed[frame],v)
-            # No longer needed: #contour,turns,vals = ImageContour.GetContour(watershed[0],v,boundingRect=boundingRect,byNeighbor=True)
-            perimeterVals,perimeterList,subContours = ImageContour.GetPerimeterByNeighborVal(watershed[frame],v,boundingRect=boundingRect,getSubContours=True)
-            subContoursAdj = [(np.array(sc)+[boundingRect[0][0],boundingRect[1][0]]).tolist() for sc in subContours] # Will need to - 0.5 to line up on an overlay
-            if len(perimeterList)>0:
-                cVLS += [ [sorted([v,perimeterVals[i]]),perimeterList[i],subContoursAdj[i]] for i in range(len(perimeterVals))]
-        cVLS.sort(cmpGen(lambda x: x[0]))
-        for i in range(len(cVLS)-1,0,-1):
-            if cVLS[i-1][0]==cVLS[i][0]:                    # if 2 subcoutours are the same,
-                cVLS[i-1][1] = min(cVLS[i-1][1],cVLS[i][1]) # keep only the one with the minimum length computation
-                del(cVLS[i])
-        cVLSByFrame.append(cVLS)
-    return cVLSByFrame
-
-def GetContourValuesLengthsAndSubContoursAndOrderOfSubContoursByFrame(watershed,allValsByFrame):
-    '''Identify every contour in the watershed segmentation for each frame as defined by each pair of cell IDs (vals).
-       Also, return the order of contours that is needed to reconstruct the border surrounding each cell.'''
-    cVLSByFrame = []
-    orderOfSCsByValueByFrame = [] # List of dicts
-    for frame in range(len(watershed)):
-        identifier=1 # unique id for each cVLS -- have to start with 1 b/c we use negatives and 0 can't be negative
-        cVLS = [] # for each subcountour: [value, length, subcontour points]
-        orderOfSCsByValue = {} # For each cellID, an ordered list of indexes to the subcontours (into cVLS) that reconstruct the contour (negative values mean the sc needs to be flipped)
-        for v in allValsByFrame[frame]:
-            boundingRect=ImageContour.GetBoundingRect(watershed[frame],v)
-            perimeterVals,perimeterList,subContours = ImageContour.GetPerimeterByNeighborVal(watershed[frame],v,boundingRect=boundingRect,getSubContours=True)
-            numSCs=len(perimeterVals)
-            subContoursAdj = [(np.array(sc)+[boundingRect[0][0],boundingRect[1][0]]).tolist() for sc in subContours] # Will need to - 0.5 to line up on an overlay
-            if len(perimeterList)>0:
-                orderOfSCsByValue[v] = []
-                for i in range(numSCs):
-                    newSC = [sorted([v,perimeterVals[i]]),perimeterList[i],subContoursAdj[i],identifier] # New Subcontours have 4 parts: valuePair,perimeter,list of xy pairs for subcontours, and identifier (arbitrary, for sorting...)
-                    matchingCVLS = [ sc for sc in cVLS if sc[0]==newSC[0] ] # match any subcoutours in cVLS so far that are for the same pair of cells
-                    matchingCVLS = [ sc for sc in matchingCVLS if totuple(sc[2][::-1])==totuple(newSC[2]) ] # Only keep subcoutours where the points match the reverse of the points in newSC
-                                    #sorted([newSC[2][0],newSC[2][-1]]) == sorted([sc[2][0],sc[2][-1]]) ]
-                                                                                                             # Should only possibly find 1 match...
-                    if matchingCVLS==[]: # New contour!
-                        cVLS.append(newSC)
-                        orderOfSCsByValue[v].append(identifier)
-                        identifier+=1
-                    else:
-                        matchingCVLS[0][1] = min(matchingCVLS[0][1],newSC[1]) # keep the minimum perimeter length...
-                        orderOfSCsByValue[v].append(-matchingCVLS[0][3]) # Negative identifier means the subcountour is backwards for this cell!
-        cVLS.sort()
-        IDs = [i[3] for i in cVLS]
-        cVLS = [i[:3] for i in cVLS] # Remove identifiers from cVLS
-        # Reindex after sorting...
-        for v in allValsByFrame[frame]:       #cmp(x,0) -> sign(x) in python
-            orderOfSCsByValue[v] = [cmp(ID,0)*IDs.index(abs(ID)) for ID in orderOfSCsByValue[v]]
-        cVLSByFrame.append(cVLS)
-        orderOfSCsByValueByFrame.append(orderOfSCsByValue)
-    return cVLSByFrame,orderOfSCsByValueByFrame
-
-        ## NOT NEEDED! KEEPING FOR REFERENCE!
-        #for i in range(len(cVLS)-1,0,-1):
-        #    for j in range(i-1,-1,-1): # Loop backwards through the sorted list of cvls's... if the value pair matches, check the endpoints (they will always be reversed for adjacent regions (always go ccw...))
-        #        if cVLS[i][0]!=cVLS[j][0]: # once we no longer match the value pair, we know there are no more matches in the list...
-        #            break
-        #        ######## VERIFY THIS ACTUALLY WORKS THE SAME WAY!!!
-        #        elif (cVLS[i][2][-1],cVLS[i][2][0]]) == (cVLS[j][2][0],cVLS[j][2][-1]):  # if 2 subcoutours are the same,
-        #            if cVLS[j][1]>cVLS[i][1]:
-        #                cVLS[j],cVLS[i] = cVLS[i],cVLS[j] #swap!
-        #            shortest = min(cVLS[j][1],cVLS[i][1])                              # keep only the one with the minimum length computation
-        #            
-        #            cVLS[j][1] = shortest
-        #            del(cVLS[i])
-        #            break
-
-def GetXYListAndPolyListFromCVLS(cVLS,allValsByFrame,orderOfSCsByValueByFrame):
-    numFrames = len(cVLS)
-    xyList = [ sorted(list(set(  tuple(pt) for c in cvlsByVal for pt in c[2]  ))) for cvlsByVal in cVLS ]
-    polyList = []
-    
-    for t in range(numFrames):
-        polyList.append({})
-        for v in allValsByFrame[t]:
-            subContours = [ ( cVLS[t][-index][2][::-1] if index<0 else cVLS[t][index][2] )
-                           for index in orderOfSCsByValueByFrame[t][v] ] # reconstruct the sc's, flipping if index is negative
-            polyList[-1][v] = [ xyList[t].index(totuple(pt)) for sc in subContours for pt in sc[:-1] ]
-            polyList[-1][v] = polyList[-1][v]+[polyList[-1][v][0]] # Tack on the first point at the end to close the loop
-            #polyList[-1][v] = removeDuplicates(polyList[-1][v])+[polyList[-1][v][0]] # Remove interior duplication...
-    return xyList,polyList
-
-def GetCVLSWithLimitedPointsBetweenNodes(cVLS,allValsByFrame,splitLength=1,fixedNumInteriorPoints=None):
-    allValues = list(set(tuple(flatten(allValsByFrame))))
-    allPairs = sorted(list(set([tuple(c[0]) for cVLSByFrame in cVLS for c in cVLSByFrame])))
-
-    if fixedNumInteriorPoints:
-        numInteriorPoints = {p:fixedNumInteriorPoints for p in allPairs}
-    else:
-        minLength = {}
-        for p in allPairs:
-            minLength[p] = min([ len(c[2]) for cVLSByFrame in cVLS for c in cVLSByFrame if tuple(c[0])==p ])
-        numInteriorPoints = {}
-        for p in allPairs:
-            numInteriorPoints[p] = minLength[p]//splitLength
-    cVLS2 = deepcopy(cVLS)
-    for cvlsByValue in cVLS2:
-        for c in cvlsByValue:
-            c[2] = limitInteriorPoints(c[2],numInteriorPoints[tuple(c[0])])
-    return cVLS2
-
-def GetXYListAndPolyListWithLimitedPointsBetweenNodes(cVLS,allValsByFrame,orderOfSCsByValueByFrame,splitLength=1,fixedNumInteriorPoints=None):
-    cVLS2 = GetCVLSWithLimitedPointsBetweenNodes(cVLS,allValsByFrame,splitLength,fixedNumInteriorPoints)
-    return GetXYListAndPolyListFromCVLS(cVLS2,allValsByFrame,orderOfSCsByValueByFrame)
-
-def ContourPlotFromImage(im,neighborPairs):
-    _=imshow_vr(im,interpolation='nearest',cmap=plt.cm.gray)
-    for i,nPair in enumerate(neighborPairs):
-        whX = np.where(  ((im[:-1,:]==nPair[0]) & (im[1:,:]==nPair[1])) |
-                         ((im[:-1,:]==nPair[1]) & (im[1:,:]==nPair[0]))  )
-        whY = np.where(  ((im[:,:-1]==nPair[0]) & (im[:,1:]==nPair[1])) |
-                         ((im[:,:-1]==nPair[1]) & (im[:,1:]==nPair[0]))  )
-        for j in range(len(whX[0])):
-            x,y = whX[1][j]-0.5 , whX[0][j]+0.5
-            _=plt.plot([x,x+1],[y,y],colors[i],linewidth=2)
-        for j in range(len(whY[0])):
-            x,y = whY[1][j]+0.5 , whY[0][j]-0.5
-            _=plt.plot([x,x],[y,y+1],colors[i],linewidth=2)
-
-def ContourPlotFromCVLS(cVLSByFrame,frame=0):
-    for cvls in cVLSByFrame[frame]:
-        cvls=np.array(cvls[2])
-        _=plt.plot( cvls[:,0], cVLS[:,1] )
-
-def GetPairLengthsAndBadSpots(pairs,pairsByFrame,cVLSByFrame,tAx):
-    pairLengths = []
-    badSpots = []
-    for pair in pairs:
-        pairLengths.append([])
-        badSpots.append([])
-        for i,cVLS in enumerate(cVLSByFrame):
-            broken = False
-            for cvls in cVLS:
-                if cvls[0]==list(pair):
-                    pairLengths[-1].append(cvls[1])
-                    broken=True
-                    break
-            if not broken:
-                pairLengths[-1].append(0)
-            elif pair not in pairsByFrame[i]:
-                badSpots[-1].append([tAx[i],pairLengths[-1][-1]])
-    return pairLengths,badSpots
-
-# If you need to use the cVLS's anyway, why compute them twice??
-# BTW, this works fine for static but DOES NOT give good connection from frame to frame
-#   1: No connection from point to point through time
-#   2: In-between points can come and go willy-nilly!!
-def MakePolygonNetworkFromWaterSeg(waterArr,minSplit=30,allValsByFrame=None,cVLS=None):
-    if allValsByFrame==None:
-        allValsByFrame = [ list(set(w.flat)) for w in waterArr ]
-    if cVLS==None:
-        cVLS = SWHelpers.GetContourValuesLengthsAndSubContoursByFrame(waterArr,allValsByFrame)
-    allPtsList = []
-    allSegsList = []
-    for t in range(len(cVLS)):
-        contours = [i[2] for i in cVLS[t]]
-        nc = len(contours)
-        lc = [len(i) for i in contours]
-        div = [i//30 for i in lc]
-        sep = [ (lc[i] if div[i]==0 else lc[i]//div[i])  for i in range(nc)]
-        #subPtsOld = [ ([] if div[i]==0 else contours[i][ ::(len(contours[i])//div[i]) ]  )
-        #          for i in range(nc) ]
-        inds = [  ( [0,lc[i]-1] if (lc[i]-1 < 2*minSplit) else
-                    range(0,lc[i]-2,sep[i])+[lc[i]-1] )
-                for i in range(nc)  ]
-        
-        subPts = [[ tuple(contours[i][j]) for j in inds[i] ] 
-                                          for i in range(nc) ]
-        allPts = sorted(list(set(flatten( subPts ))))
-        subPtsInds = [[ allPts.index(j) for j in i ] 
-                                        for i in subPts ]
-        allSegsInds = flatten([ zip(i[:-1],i[1:]) for i in subPtsInds ])
-        allPtsList.append(allPts)
-        allSegsList.append(allSegsInds)
-        
-    return allPtsList,allSegsList
-
 def MultNumOrNone(x,y):
     if x==None or y==None:
         return None # None's -> None's
@@ -521,7 +360,8 @@ def GetNeighborValues(d,wv,ind):
         allVals = []
         for v in wv:
             n=neighbors[ind][v-2]
-            if n!=None:  allVals+=n
+            if n!=None:
+                allVals+=n
         allVals = list(set(allVals).difference(wv))
         return allVals
     else:
@@ -546,8 +386,8 @@ def OutlineMasksPlot(arr,waterArr,cellID,t=0):
     plt.imshow(bright,cmap=plt.cm.gray)
     
     for i in range(1,5):
-        wo = ndimage.binary_dilation(wv,iterations=i) -  \
-             ndimage.binary_erosion(wv,iterations=i)
+        wo = ( ndimage.binary_dilation(wv,iterations=i) -
+               ndimage.binary_erosion(wv,iterations=i)  )
         wh=np.where(wo)
         bright = np.zeros_like(arr[t])
         bright[wh]=arr[t][wh]
@@ -560,8 +400,8 @@ def DilateMinusErode(binaryArr,structure=None,iterations=1):
         print 'You must specify at least one iteration!!!'
         return
     else:
-        return ndimage.binary_dilation(binaryArr,structure=structure,iterations=iterations) -  \
-                ndimage.binary_erosion(binaryArr,structure=structure,iterations=iterations)
+        return ( ndimage.binary_dilation(binaryArr,structure=structure,iterations=iterations) -
+                 ndimage.binary_erosion(binaryArr,structure=structure,iterations=iterations)  )
 
 def GetCellVolume(waterArr,cellID,structure=None,iterations=0):
     sums = []
@@ -587,7 +427,8 @@ def GetTotalIntegratedBrightness(arr,waterArr,cellID,structure=None,iterations=0
     return np.array(sums)
 
 def GetEdgeIntegratedBrightness(arr,waterArr,cellID,thickness=2,skip3DTopAndBottom=False):
-    '''Sum the brightness of the original image under the edges of the final segmented region, with a specificed thickness (even integers only) (2D+time or 3D+time)'''
+    '''Sum the brightness of the original image under the edges of the final segmented region,
+       with a specificed thickness (even integers only) (2D+time or 3D+time)'''
     if thickness%2==1 or thickness<2:
         print 'thickness parameter must be an even integer > 0!'
         return
@@ -598,10 +439,16 @@ def GetEdgeIntegratedBrightness(arr,waterArr,cellID,thickness=2,skip3DTopAndBott
         structure = np.array([ [[0,1,0],[1,1,1],[0,1,0]] ],np.bool)
         return GetTotalIntegratedBrightness(arr,waterArr,cellID,structure=structure,iterations=iterations)
     else:
-        'You can only use skip3DTopAndBottom=True on a 4D array!'
+        print 'You can only use skip3DTopAndBottom=True on a 4D array!'
         return
 
 def GetVolumesAndIntegratedBrightnesses(testArrays,waterArr,cellID,thickness,skip3DBottomAndTop=True):
+    '''Takes testArrays (one or more sets of raw data), waterArr (segmentation data), and cellID to make measurements of a cell
+       Returns the volume under the cell and the volume of the xy-rim around the cell with a certain thicknes
+           and for each of these for each testArray: the integrated intensity and average intensity per pixel of the raw array under these same regions
+       again, testArrays is a list of arrays and the return values: "intensities","edgeIntensities","intensitiesPerPixel","edgeIntensitiesPerPixel"
+           all return lists of measurements for each testArray
+       Uses GetCellVolume in both direct mode (iterations=0) and edge mode (runs dilate-erode <iterations> times) to get a band around the edge'''
     volume = GetCellVolume(waterArr,cellID)
     volumeEdge = GetCellVolume(waterArr,cellID,structure=[[[0,1,0],[1,1,1],[0,1,0]]],iterations=thickness//2)
     
@@ -622,6 +469,7 @@ def GetVolumesAndIntegratedBrightnesses(testArrays,waterArr,cellID,thickness,ski
 # An old function from CellFunctions.py for plotting some weird gradient angles...
 # Just put it here to keep from losing the code
 #####################################################################
+'''
 def GradientAnglePlot(f='/media/sda1/Documents and Settings/Owner/My Documents/VIIBRE--ScarHealing/ActiveData/Stack_Zproject_GBR_DC.gif'):
     r=VolumeGif.LoadMonolithicGif(f)[0]
     
@@ -641,4 +489,4 @@ def GradientAnglePlot(f='/media/sda1/Documents and Settings/Owner/My Documents/V
     imshow(n.arctan(pdy/pdx))
     subplot(1,numSub,5)
     imshow(ndimage.convolve(test,n.array([[-1,0,1],[0,0,0],[1,0,-1]])))
-
+'''
