@@ -1,5 +1,7 @@
 # This should import everything relevant:
 from SeedWaterSegmenter import *
 import SWHelpers
+from ImageContour import SubContourTools
+from _version import *
 
 __all__ = ['SeedWaterSegmenter','SWHelpers']
